import serial, time, math, commands, sys, os

GETVERSION  =   1		#Commands with One byte argument (41 to 80) 
READADC     =   2
SETLED		=  41		#Commands with One byte argument (41 to 80) 
READCHAN	=  42

linux_list = ['/dev/ttyACM0','/dev/ttyACM1']	#Serial devices to search for hardware.  

def open(dev = None):
	'''
	If EYES hardware in found, returns an instance of 'Eyes', else returns None.
	'''
	obj = Cmeter()
	if obj.fd != None:
		return obj
	print 'Could not find Solar cell hardware'
	print 'Check the connections.'

BUFSIZE     = 100       	
BAUDRATE = 38400			# Serial communication
MAXV = 5.0
AVCC = 4.896					  # 7805 output for this unit

class Cmeter:
	fd = None								# init should fill this
	msg = ''

	def __init__(self, dev = None):
		"""
		Searches for Colorimeter hardware on ACM devices by reading the version string.
		"""
		self.adcsize = 2;
		
		if os.name == 'nt':	
			device_list = []
			for k in range(1,255):
				s = 'COM%d'%k
				device_list.append(s)
			for k in range(1,11):
				device_list.append(k)
		else:
			device_list = []    # gather unused ones from the linux_list
			for dev in linux_list:
				res = commands.getoutput('lsof -t '+ str(dev))	
				if res == '': 
					device_list.append(dev)
		
		for dev in device_list:
			try:
				handle = serial.Serial(dev, BAUDRATE, stopbits=1, timeout = 0.3, parity=serial.PARITY_EVEN)
			except:
				continue
			self.msg += 'Port %s is existing '%dev
			if handle.isOpen() != True:
				print 'but could not open'
				continue
			self.msg += ' and opened. '
			handle.flush()
			time.sleep(.5)
			while handle.inWaiting() > 0 :
				handle.flushInput()
			handle.write(chr(GETVERSION))
			res = handle.read(1)
			ver = handle.read(5)		# 5 character version number
			if ver[:2] == 'cm':
				self.device = dev
				self.fd = handle
				self.version = ver
				handle.timeout = 4.0	# r2rtime on .7 Hz require this
				self.msg += 'Found EYES version %s'%ver
				return 
			else:
				handle.close()
		print self.msg
		self.fd = None
#------------------------------------------------------------------------------------

	def dwrite(self,ch):
		self.fd.write(ch)
		time.sleep(0.01)		#MCP2200 to ATmega transfer has no handshake

	def set_led(self, n):
		'''
		Set the selected LED, from 1 to 20 only
		'''
		if n < 1 or n > 20:
			print 'LED must be from 1 to 20'
			return
		self.dwrite(chr(SETLED))
		self.dwrite(chr(n))
		res = self.fd.read(1)
		if res != 'D':
			print 'SETLED error ', res		

	def read_chan(self, n):
		'''
		Turn on the selected LED, and read the ADC input voltage.
		'''
		if n < 1 or n > 20:
			print 'Channel must be from 1 to 20'
			return
		self.dwrite(chr(READCHAN))
		self.dwrite(chr(n))
		res = self.fd.read(1)
		if res != 'D':
			print 'READCHAN error ', res		
		res = self.fd.read(2)
		iv = ord(res[0]) | (ord(res[1]) << 8)
		v = AVCC * iv /1023
		return v

	def read_adc(self):
		'''
		read the ADC input voltage.
		'''
		self.dwrite(chr(READADC))
		res = self.fd.read(1)
		if res != 'D':
			print 'READADC error ', res		
		res = self.fd.read(2)
		iv = ord(res[0]) | (ord(res[1]) << 8)
		print iv
		v = AVCC * iv /1023
		return v

	def get_version(self):
		self.dwrite(chr(GETVERSION))
		res = self.fd.read(1)
		if res != 'D':
			p.msg = 'GETVERSION ERROR' + res
			return
		ver = self.fd.read(5)
		return ver


#--------------------------------- may go to eyeutils.py ------------------------------
	def grace(self, data, xlab = '', ylab = '', title = ''):
		'''
		Input data is of the form, [ [x1,y1], [x2,y2],....] where x and y are vectors
		'''
		try:
			import pygrace
			pg = pygrace.grace()
			for xy in data:
				pg.plot(xy[0],xy[1])
				pg.hold(1)				# Do not erase the old data
			pg.xlabel(xlab)
			pg.ylabel(ylab)
			pg.title(title)
			return True
		except:
			return False


