from Tkinter import *
import expeyes.solarlib, expeyes.eyeplot, time, math, sys

WIDTH = 600
HEIGHT = 400
MAXV   = 5.0
MINWL = 390.0
MAXWL = 1000.0


wavelens = [365., 380., 400., 435., 450., 490., 535., 600., 660., 690.,\
			720., 760., 800., 850., 890., 910., 940., 970., 1020.,1060.]

oldp = [0.645, 3.467, 12.915, 12.154, 10.288, 7.870, 2.909, 1.875, 2.285, 6.429,\
		    6.384, 6.590,  7.006,  9.875,  4.607, 2.387, 4.135, 1.849, 0.612, 0.689]

pinvals  = [2.947, 4.344, 9.937, 19.751, 11.971, 8.430, 4.950, 3.863, 10.844, 5.326,\
			6.044, 8.379, 8.294,  8.264,  5.052, 3.385, 5.242, 2.380,  5.364, 4.932]

NLED = len(wavelens)
results = [0.0] * NLED

def	msg(s, col = 'blue'):
	msgwin.config(text = s, fg=col)

def read_data():
	global results, wavelen, NLED
	maxval = 0.1
	print '---------'
	for n in range(20):
		res = p.read_chan(n+1)   	# 1 to 20
		res2 = res / pinvals[n]     #* 0.808 * wavelens[n]
		results[n-1] = res2
		if res2 > maxval: maxval = res2
		print '%2d %8.2f %8.3f %8.2f'%(n+1,res, pinvals[n],res2)	
	g.setWorld(MINWL, 0, MAXWL, maxval*1.1,'wave length(nm)', 'Volt (V)')
	g.delete_lines()
	g.line(wavelens, results)

def save_data():
	global results, wavelen, NLED
	try:
		fn = filename.get()
	except:
		fn = 'solar.dat'
	f = open(fn, 'w')
	for ch in range(NLED):
		s = '%5.0f  %5.3f\n'%(wavelens[ch], results[ch])
		f.write(s)
	msgwin.config(text = 'Data Saved')

def single_led():
	try:
		n = int(LED.get())
	except:
		msg('LED from 1 to 20','red')
		return
	if n < 1 or n > 20:
		msg('LED from 1 to 20','red')
		return
	p.set_led(n)
	v = p.read_adc()
	sr = v/pinvals[n-1]
	s = 'LED# %d(w =%4d nm, Pin = %5.3f): Vout = %5.2f V  Spec.Response = %5.2f'\
			%(n,wavelens[n-1], pinvals[n-1],v, sr)
	msg(s)
	return sr

root = Tk()
top = Frame(root)
top.pack()
g = expeyes.eyeplot.graph(top, width=WIDTH, height=HEIGHT)	# make plot
g.setWorld(MINWL, 0, MAXWL, 5.,'wave length(nm)', 'Volt (V)')

cf = Frame(root, width = WIDTH, height = 10)
cf.pack(side=TOP,  fill = BOTH, expand = 1)

b1 = Button(cf, text = 'Measure', command = read_data)
b1.pack(side = LEFT, anchor = N)
b3 = Button(cf, text = 'SAVE to', command = save_data)
b3.pack(side = LEFT, anchor = N)
filename = StringVar()
e1 =Entry(cf, width=15, bg = 'white', textvariable = filename)
filename.set('solar.dat')
e1.pack(side = LEFT)

LED = StringVar()
e2 =Entry(cf, width=5, bg = 'white', textvariable = LED)
LED.set('10')
e2.pack(side = LEFT)
b2 = Button(cf, text = 'Single', command = single_led)
b2.pack(side = LEFT, anchor = N)


b5 = Button(cf, text = 'QUIT', command = sys.exit)
b5.pack(side = RIGHT, anchor = N)
mf = Frame(root, width = WIDTH, height = 10)
mf.pack(side=TOP,  fill = BOTH, expand = 1)
msgwin = Label(mf,text='', fg = 'blue')
msgwin.pack(side=LEFT)

p=expeyes.solarlib.open()
if p == None:
	msg('Could not Find Hardware. Bad connection or another program using it','red')
	#Recon.pack(side=LEFT)
else:
	root.title('Found ?? on %s (firmware: %s)'%(p.device,p.get_version()))
root.mainloop()

